@@ -6,7 +6,7 @@
  */
 
 const FHC_WINDOW_MANAGE  = { path:"popup/tableview/popup-big.html", width:1000, height:500, type:"popup", currentId: -1 };
-const FHC_WINDOW_OPTIONS = { path:"popup/options/options.html",     width: 575, height:430, type:"popup", currentId: -1 };
+const FHC_WINDOW_OPTIONS = { path:"popup/options/options.html",     width: 575, height:450, type:"popup", currentId: -1 };
 const FHC_WINDOW_ABOUT   = { path:"popup/about/about.html",         width: 600, height:300, type:"popup", currentId: -1 };
 const FHC_WINDOW_IMPORT  = { path:"popup/importexport/import.html", width: 350, height:250, type:"popup", currentId: -1 };
 const FHC_WINDOW_EXPORT  = { path:"popup/importexport/export.html", width: 350, height:250, type:"popup", currentId: -1 };
@@ -345,9 +345,10 @@ class WindowUtil {
         const edgeMargin = 10;
 
         const winRect = document.querySelector(queryContainerElement).getBoundingClientRect();
+        const mnuWrapper = document.getElementById('context-menu-wrapper');
 
         document.getElementById('context-menu-container').style.display = 'block';
-        const menuRect = document.getElementById('context-menu-wrapper').getBoundingClientRect();
+        const menuRect = mnuWrapper.getBoundingClientRect();
 
         // get the mouse position and apply an offset to get the mouse-pointer on the first item
         let x = Math.max(edgeMargin, event.pageX - 60);
@@ -363,12 +364,16 @@ class WindowUtil {
             y = winRect.height - (menuRect.height + edgeMargin);
         }
 
-        const mnu = document.getElementById('context-menu-wrapper');
-        mnu.style.top = y + "px";
-        mnu.style.left = x + "px";
+        mnuWrapper.style.top = y + "px";
+        mnuWrapper.style.left = x + "px";
 
         // trigger the transition
-        document.getElementById('context-menu-wrapper').classList.add('show');
+        mnuWrapper.classList.add('show');
+
+        mnuWrapper.addEventListener("mouseleave", function hide() {
+            mnuWrapper.classList.remove('show');
+            mnuWrapper.removeEventListener("mouseleave", hide);
+        });
     }
 
     static hideContextMenuOnClick(event) {
@@ -383,6 +388,10 @@ class WindowUtil {
         window.setTimeout(()=>{document.getElementById('context-menu-container').style.display = 'none';}, 800);
     }
 
+    static isContextMenuShown() {
+        return document.getElementById('context-menu-wrapper').classList.contains('show');
+    }
+
     /**
      * Convert HTML to a human readable text by stripping all tags but keeping the same look by adding breaks
      * and asterix's for lists.
