@@ -6,21 +6,56 @@
  */
 'use strict';
 
-/**
- * Caveat:
- * does not detect switching/focusing window, with multiple windows each window has an active tab and the
- * context menu will reflect only the state of the last activated (switched) tab on that particular window.
- */
-
 const IS_FIREFOX = typeof browser.runtime.getBrowserInfo === 'function';
 console.log("IS_FIREFOX = " + IS_FIREFOX);
 
-browser.tabs.onActivated.addListener(handleActivated);
+browser.runtime.onMessage.addListener(receiveContextEvents);
 
-// initially set the EditorFieldRestoreMenu for the current active tab
+function receiveContextEvents(fhcEvent, sender, sendResponse) {
+    if (fhcEvent.eventType && fhcEvent.eventType === 888 && fhcEvent.contextmenuAvailChanged) {
+        // remove the context menu and rebuild from scratch
+        const promisesArray = [];
+        while (CONTEXT_FIELDS_MENUITEM_IDS.length > 0) {
+            let itemId = CONTEXT_FIELDS_MENUITEM_IDS.pop();
+            promisesArray.push(browserMenusRemove(itemId));
+        }
+        Promise.all(promisesArray).then(()=>{
+            browser.storage.local.get({
+                prefContextmenuAvail: "page"
+            }).then(res => {
+                _initContextMenu(res.prefContextmenuAvail);
+                _initBrowserActionSubmenu();
+            });
+        });
+    }
+}
+
+
+// keep track of current menu, activating another tab on another window triggers both
+// tab.onActivated and windows.onFocusChanged which triggers both event handlers
+const CUR_MENU = {
+    windowId: -1,
+    tabId: -1,
+    host: ''
+};
+browser.tabs.onActivated.addListener(handleTabActivated);
+browser.tabs.onUpdated.addListener(handleTabUpdated);
+browser.windows.onFocusChanged.addListener(handleWindowFocusChanged);
+
+const debounce = (fn, time) => {
+    let timeout;
+    return function() {
+        const functionCall = () => fn.apply(this, arguments);
+        clearTimeout(timeout);
+        timeout = setTimeout(functionCall, time);
+    }
+};
+
+
+// initially set the EditorFieldRestoreMenu for the current active window and tab
 setTimeout(()=>{ updateEditorFieldRestoreMenuForActiveTab(); }, 1500);
 
-// create the comtext menus
+// create the context menus
 initBrowserMenus();
 
 // set the preferred shortcut keys and add a shortcutKey listener
@@ -29,86 +64,90 @@ browser.commands.onCommand.addListener(handleShortcutKeys);
 
 
 function updateEditorFieldRestoreMenuForActiveTab() {
-    browser.tabs.query({active: true}).then(tabInfo=>{
-        if (tabInfo.length > 0) {
-            tabInfo.forEach(tab => {
-                updateEditorFieldRestoreMenu(tab.url);
+    browser.windows.getCurrent({populate: true}).then(tabInfo=>{
+        // console.log('Active window is ' + tabInfo.id);
+        if (tabInfo.tabs.length > 0) {
+            tabInfo.tabs.forEach(tab => {
+                if (tab.active) {
+                    // console.log('Active tab is ' + tab.id);
+                    debouncedUpdateEditorFieldRestoreMenu(tab.windowId, tab.id, tab.url);
+                }
             });
         }
     });
 }
 
-function handleActivated(activeInfo) {
-    // console.log("Tab " + activeInfo.tabId + " was activated");
-    // create submenu-items for multiline restore
-    updateEditorFieldRestoreMenuOnTabActivation(activeInfo.tabId);
+function handleWindowFocusChanged(windowId) {
+    if (windowId > 0) {
+        // console.log("### Window " + windowId + " now has the focus! ###");
+        updateEditorFieldRestoreMenuForActiveTab();
+    }
 }
 
-function updateEditorFieldRestoreMenuOnTabActivation(tabId, attempt = 1) {
+function handleTabUpdated(tabId, changeInfo, tab) {
+    if (changeInfo.status === 'complete') {
+        updateEditorFieldRestoreMenuOnTabActivation(tab.windowId, tab.id);
+    }
+}
+
+function handleTabActivated(activeInfo) {
+    // console.log("### Tab " + activeInfo.tabId + " was activated for window " + activeInfo.windowId + " ###");
+    // create submenu-items for multiline restore
+    updateEditorFieldRestoreMenuOnTabActivation(activeInfo.windowId, activeInfo.tabId);
+}
+
+function updateEditorFieldRestoreMenuOnTabActivation(windowId, tabId, attempt = 1) {
+    // console.log("Update editorFieldRestoreMenu for Tab " + tabId);
     browser.tabs.get(tabId).then(tabInfo => {
         if (tabInfo.status === 'loading' || ('about:blank' === tabInfo.url && attempt<=10)) {
+            // console.log('TabId ' + tabId + ' has not finished loading, trying again in 500ms');
             setTimeout(() => {
-                updateEditorFieldRestoreMenuOnTabActivation(tabId, ++attempt);
+                updateEditorFieldRestoreMenuOnTabActivation(windowId, tabId, ++attempt);
             }, 500);
         } else {
-            // console.log('TabId ' + tabId+ ' was activated and has url: ' + tabInfo.url);
-            updateEditorFieldRestoreMenu(tabInfo.url);
+            // console.log('TabId ' + tabId + ' was activated and has url: ' + tabInfo.url);
+            debouncedUpdateEditorFieldRestoreMenu(tabInfo.windowId, tabInfo.id, tabInfo.url);
         }
     });
 }
 
 const MAX_LENGTH_EDITFIELD_ITEM = 35;
-const editorFieldsMenuItemsIds = [];
+const EDITOR_FIELDS_MENUITEM_IDS = [];
+let contextAnonMenuItemNo = 10000;
+const CONTEXT_FIELDS_MENUITEM_IDS = [];
 
-function updateEditorFieldRestoreMenu(url) {
+function updateEditorFieldRestoreMenu(windowId, tabId, url) {
+    // console.log('>>> updateEditorFieldRestoreMenu for window ' + windowId + ' and tab with url ' + url);
     if (url.includes('moz-extension://')) {
         // skip popup windows
         return;
     }
-    const hostname = MiscUtil.getHostnameFromUrlString(url);
 
-    removeCurrentMenuItems(editorFieldsMenuItemsIds)
+    const hostname = MiscUtil.getHostnameFromUrlString(url);
+    if (CUR_MENU.windowId === windowId && CUR_MENU.tabId === tabId && CUR_MENU.host === hostname) {
+        // console.log('!! skip duplicate call to updateEditorFieldRestoreMenu() for window ' + windowId + ' and tab with host ' + hostname);
+        return;
+    }
+    CUR_MENU.windowId = windowId;
+    CUR_MENU.tabId = tabId;
+    CUR_MENU.host = hostname;
+
+    removeCurrentMenuItems(EDITOR_FIELDS_MENUITEM_IDS)
     .then(() => {
         return getEditorFieldsByHostname(hostname, 10);
     }).then(hostnameItemsArray => {
-        hostnameItemsArray.forEach(item => {editorFieldsMenuItemsIds.push(item);});
+        hostnameItemsArray.forEach(item => {EDITOR_FIELDS_MENUITEM_IDS.push(item);});
         return hostnameItemsArray;
     }).then(hostnameItemsArray => {
         return getEditorFieldsByLastused(hostname, 10, hostnameItemsArray);
     }).then(lastusedItemsArray => {
-        lastusedItemsArray.forEach(item => {editorFieldsMenuItemsIds.push(item);});
+        lastusedItemsArray.forEach(item => {EDITOR_FIELDS_MENUITEM_IDS.push(item);});
     }).then(()=>{
         // editorFieldsMenuItemsIds.forEach(item => { console.log('- ' + item.type + ' ' + item.pKey + '  ' + item.value); });
-        return addNewMenuItems(editorFieldsMenuItemsIds);
+        return addNewMenuItems(EDITOR_FIELDS_MENUITEM_IDS);
     });
 }
-
-// function updateEditorFieldRestoreMenu(tabId) {
-//     browser.tabs.get(tabId).then(tabInfo => {
-//         if (tabInfo.status === 'loading') {
-//             // console.log('TabId ' + tabId + ' not completely loaded yet, retry getting tabInfo in 1 sec...');
-//             setTimeout(()=>{ updateEditorFieldRestoreMenu(tabId); }, 1000);
-//         } else {
-//             const hostname = MiscUtil.getHostnameFromUrlString(tabInfo.url);
-//             // console.log('TabId ' + tabId + ' was activated and has url: ' + tabInfo.url + '  (' + hostname + ')');
-//
-//             removeCurrentMenuItems(editorFieldsMenuItemsIds)
-//             .then(() => {
-//                 return getEditorFieldsByHostname(hostname, 10);
-//             }).then(hostnameItemsArray => {
-//                 hostnameItemsArray.forEach(item => {editorFieldsMenuItemsIds.push(item);});
-//                 return hostnameItemsArray;
-//             }).then(hostnameItemsArray => {
-//                 return getEditorFieldsByLastused(hostname, 10, hostnameItemsArray);
-//             }).then(lastusedItemsArray => {
-//                 lastusedItemsArray.forEach(item => {editorFieldsMenuItemsIds.push(item);});
-//             }).then(()=>{
-//                 // editorFieldsMenuItemsIds.forEach(item => { console.log('- ' + item.type + ' ' + item.pKey + '  ' + item.value); });
-//                 return addNewMenuItems(editorFieldsMenuItemsIds);
-//             });
-//         }
-//     });
-// }
+let debouncedUpdateEditorFieldRestoreMenu = debounce(updateEditorFieldRestoreMenu, 250);
 
 function addNewMenuItems(menuItemsIds) {
     return new Promise((resolve, reject) => {
@@ -331,10 +370,20 @@ function onMenuCreated() {
 
 
 function initBrowserMenus() {
+    const gettingPref = browser.storage.local.get({
+        prefContextmenuAvail: "page"
+    });
+    gettingPref.then(res => {
+        const contextmenuAvail = res.prefContextmenuAvail;
+        _initToolsMenu();
+        _initContextMenu(contextmenuAvail);
+        _initBrowserActionSubmenu();
+    });
+}
+
+function _initToolsMenu() {
     /*
      * Create the Tools context menu items.
-     *
-     * ===============================(tools_menu)================================
      */
     browserMenusCreate({
         id: "FHCToolsParentMenu",
@@ -365,153 +414,169 @@ function initBrowserMenus() {
             "32": "/theme/icons/menu/32/preferences.png"
         }
     }, onMenuCreated);
-    /* =============================(tools_menu end)============================== */
-
+}
 
+function _initContextMenu(contextmenuAvail) {
     /*
      * Create the right-click context menu.
      * Hide the menu separators for the browser-action, we may only show 6 items
      * including the separators.
-     *
-     * ==============================(context menu)===============================
      */
-    browserMenusCreate({
+    const contextAll     = [];
+    const contextEdFr    = [];
+    const contextEdFrBra = ["browser_action"];
+
+    // Empty the array
+    CONTEXT_FIELDS_MENUITEM_IDS.splice(0, CONTEXT_FIELDS_MENUITEM_IDS.length);
+
+    if (contextmenuAvail === 'page') {
+        contextAll.push("all");
+        contextEdFr.push("page");
+        contextEdFrBra.push("page");
+    } else {
+        contextAll.push("frame", "browser_action",  "page_action");
+        if (contextmenuAvail === 'editfields') {
+            contextAll.push("editable");
+            contextEdFr.push("editable");
+            contextEdFrBra.push("editable");
+        }
+    }
+
+    browserContextMenusCreate({
         id: "manage",
         title: browser.i18n.getMessage("contextMenuItemManageHistory"),
-        contexts: ["all"],
+        contexts: contextAll,
         icons: {
             "16": "/theme/icons/fhc-16.png",
             "32": "/theme/icons/fhc-32.png"
         }
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         type: "separator",
-        contexts: ["page","editable","frame"]
+        contexts: contextEdFr
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         id: "restoreEditorField",
         title: browser.i18n.getMessage("contextMenuItemRestoreEditorField"),
-        contexts: ["all"],
+        contexts: contextAll,
         icons: {
             "16": "/theme/icons/menu/16/refresh.png",
             "32": "/theme/icons/menu/32/refresh.png"
         }
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         type: "separator",
-        contexts: ["page","editable","frame"]
+        contexts: contextEdFr
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         id: "fillMostRecent",
         title: browser.i18n.getMessage("contextMenuItemFillMostRecent"),
-        contexts: ["all"],
+        contexts: contextAll,
         icons: {
             "16": "/theme/icons/menu/16/fillfields.png",
             "32": "/theme/icons/menu/32/fillfields.png"
         }
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         id: "fillMostUsed",
         title: browser.i18n.getMessage("contextMenuItemFillMostUsed"),
-        contexts: ["all"],
+        contexts: contextAll,
         icons: {
             "16": "/theme/icons/menu/16/fillfields.png",
             "32": "/theme/icons/menu/32/fillfields.png"
         }
     }, onMenuCreated);
-// do not show this menu-item for page_action, only 5 items are shown
-    browserMenusCreate({
+    // do not show this menu-item for page_action, only 5 items are shown
+    browserContextMenusCreate({
         id: "clearFields",
         title: browser.i18n.getMessage("contextMenuItemClearFields"),
-        contexts: ["page","editable","frame","browser_action"],
+        contexts: contextEdFrBra,
         icons: {
             "16": "/theme/icons/menu/16/emptyfields.png",
             "32": "/theme/icons/menu/32/emptyfields.png"
         }
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         type: "separator",
-        contexts: ["page","editable","frame"]
+        contexts: contextEdFr
     }, onMenuCreated);
     /*
      * Remainder only for page_action (max 6 are shown for browser-action).
-     * ============================(context menu page)============================
      */
-    browserMenusCreate({
+    browserContextMenusCreate({
         id: "showformfields",
         title: browser.i18n.getMessage("contextMenuItemShowformfields"),
-        contexts: ["page","editable","frame"],
+        contexts: contextEdFr,
         icons: {
             "16": "/theme/icons/menu/16/showfields.png",
             "32": "/theme/icons/menu/32/showfields.png"
         }
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         type: "separator",
-        contexts: ["page","editable","frame"]
+        contexts: contextEdFr
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         id: "submenuInfo",
         title: browser.i18n.getMessage("menuItemInfoSubmenu"),
-        contexts: ["page","editable","frame"],
+        contexts: contextEdFr,
         icons: {
             "16": "/theme/icons/menu/16/submenu.png",
             "32": "/theme/icons/menu/32/submenu.png"
         }
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         id: "helpoverview",
         parentId: "submenuInfo",
         title: browser.i18n.getMessage("menuItemHelpOverview"),
-        contexts: ["page","editable","frame"],
+        contexts: contextEdFr,
         icons: {
             "16": "/theme/icons/menu/16/help.png",
             "32": "/theme/icons/menu/32/help.png"
         }
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         id: "releasenotes",
         parentId: "submenuInfo",
         title: browser.i18n.getMessage("menuItemHelpReleasenotes"),
-        contexts: ["page","editable","frame"],
+        contexts: contextEdFr,
         icons: {
             "16": "/theme/icons/menu/16/releasenotes.png",
             "32": "/theme/icons/menu/32/releasenotes.png"
         }
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         id: "about",
         parentId: "submenuInfo",
         title: browser.i18n.getMessage("menuItemHelpAbout"),
-        contexts: ["page","editable","frame"],
+        contexts: contextEdFr,
         icons: {
             "16": "/theme/icons/menu/16/about.png",
             "32": "/theme/icons/menu/32/about.png"
         }
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         type: "separator",
-        contexts: ["page","editable","frame"]
+        contexts: contextEdFr
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         id: "preferences",
         title: browser.i18n.getMessage("contextMenuItemOptions"),
-        contexts: ["page","editable","frame"],
+        contexts: contextEdFr,
         icons: {
             "16": "/theme/icons/menu/16/preferences.png",
             "32": "/theme/icons/menu/32/preferences.png"
         }
     }, onMenuCreated);
-    /* ==========================(context menu page end)========================== */
+}
 
+function _initBrowserActionSubmenu() {
     /*
      * Browser-action (right-click on icon in menu-bar) may only show 6 items,
      * Page-action (right-click on icon in address-bar) may only show 5 items,
      * put remainder in a submenu (browser.menus.ACTION_MENU_TOP_LEVEL_LIMIT)
-     *
-     * =========================(browser_action submenu)==========================
      */
-    browserMenusCreate({
+    browserContextMenusCreate({
         id: "submenuExtra",
         title: browser.i18n.getMessage("contextMenuItemRestoreEditorFieldSubmenuMore"),
         contexts: ["browser_action", "page_action"],
@@ -520,7 +585,7 @@ function initBrowserMenus() {
             "32": "/theme/icons/menu/32/submenu.png"
         }
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         id: "clearFieldsPA",
         parentId: "submenuExtra",
         title: browser.i18n.getMessage("contextMenuItemClearFields"),
@@ -530,12 +595,12 @@ function initBrowserMenus() {
             "32": "/theme/icons/menu/32/emptyfields.png"
         }
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         parentId: "submenuExtra",
         type: "separator",
         contexts: ["page_action"]
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         id: "showformfieldsBA",
         parentId: "submenuExtra",
         title: browser.i18n.getMessage("contextMenuItemShowformfields"),
@@ -545,12 +610,12 @@ function initBrowserMenus() {
             "32": "/theme/icons/menu/32/showfields.png"
         }
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         parentId: "submenuExtra",
         type: "separator",
         contexts: ["all"]
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         id: "helpoverviewBA",
         parentId: "submenuExtra",
         title: browser.i18n.getMessage("menuItemHelpOverview"),
@@ -560,7 +625,7 @@ function initBrowserMenus() {
             "32": "/theme/icons/menu/32/help.png"
         }
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         id: "releasenotesBA",
         parentId: "submenuExtra",
         title: browser.i18n.getMessage("menuItemHelpReleasenotes"),
@@ -570,7 +635,7 @@ function initBrowserMenus() {
             "32": "/theme/icons/menu/32/releasenotes.png"
         }
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         id: "aboutBA",
         parentId: "submenuExtra",
         title: browser.i18n.getMessage("menuItemHelpAbout"),
@@ -580,12 +645,12 @@ function initBrowserMenus() {
             "32": "/theme/icons/menu/32/about.png"
         }
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         parentId: "submenuExtra",
         type: "separator",
         contexts: ["all"]
     }, onMenuCreated);
-    browserMenusCreate({
+    browserContextMenusCreate({
         id: "preferencesBA",
         parentId: "submenuExtra",
         title: browser.i18n.getMessage("contextMenuItemOptions"),
@@ -595,11 +660,9 @@ function initBrowserMenus() {
             "32": "/theme/icons/menu/32/preferences.png"
         }
     }, onMenuCreated);
-    /* ===========================(browser_action end)============================ */
 }
 
 
-
 function showformfields(tabId) {
     // send without checking response
     //console.log('Sending a message to tab ' + tabId);
@@ -724,19 +787,30 @@ getBrowserMenusOnClickedHandler().addListener(function(info, tab) {
  * Cross browser (Firefox, Chrome) create menus.
  */
 function browserMenusCreate(menuProperties, onMenuCreated) {
-    if (IS_FIREFOX) {
-        return browser.menus.create(menuProperties, onMenuCreated);
-    } else {
-        // strip unsupported icons
-        delete menuProperties['icons'];
-        // skip unsupported "tools_menu" context
-        if (menuProperties.contexts.includes("tools_menu")) {
-            return null;
+    if (menuProperties.contexts.length) {
+        if (IS_FIREFOX) {
+            return browser.menus.create(menuProperties, onMenuCreated);
+        } else {
+            // strip unsupported icons
+            delete menuProperties['icons'];
+            // skip unsupported "tools_menu" context
+            if (menuProperties.contexts.includes("tools_menu")) {
+                return null;
+            }
+            return chrome.contextMenus.create(menuProperties, onMenuCreated);
         }
-        return chrome.contextMenus.create(menuProperties, onMenuCreated);
     }
 }
 
+function browserContextMenusCreate(menuProperties, onMenuCreated) {
+    if (!menuProperties.id) {
+        menuProperties.id = 'noname' + (++contextAnonMenuItemNo);
+    }
+    CONTEXT_FIELDS_MENUITEM_IDS.push(menuProperties.id);
+    browserMenusCreate(menuProperties, onMenuCreated);
+}
+
+
 /**
  * Cross browser (Firefox, Chrome) remove menu.
  */
@@ -763,33 +837,46 @@ function initShortcutKeys() {
 
 function handleShortcutKeys(command) {
     // console.log("Command! " + command);
-    switch (command) {
-        case "open_fhc":
-            WindowUtil.createOrFocusWindow(FHC_WINDOW_MANAGE);
-            break;
+    OptionsUtil.getShortcutKeysEnablePrefs().then(res => {
+        switch (command) {
+            case "open_fhc":
+                if (res.prefShortcutKeys['open_fhc_enable']) {
+                    WindowUtil.createOrFocusWindow(FHC_WINDOW_MANAGE);
+                }
+                break;
 
-        case "toggle_display_fields":
-            browser.tabs.query({active: true, currentWindow: true}).then(tabInfo => {
-                showformfields(tabInfo[0].id);
-            });
-            break;
+            case "toggle_display_fields":
+                if (res.prefShortcutKeys['toggle_display_fields_enable']) {
+                    browser.tabs.query({active: true, currentWindow: true}).then(tabInfo => {
+                        showformfields(tabInfo[0].id);
+                    });
+                }
+                break;
 
-        case "fill_recent":
-            browser.tabs.query({active: true, currentWindow: true}).then(tabInfo => {
-                fillformfields(tabInfo[0].id, "fillMostRecent");
-            });
-            break;
+            case "fill_recent":
+                if (res.prefShortcutKeys['fill_recent_enable']) {
+                    browser.tabs.query({active: true, currentWindow: true}).then(tabInfo => {
+                        fillformfields(tabInfo[0].id, "fillMostRecent");
+                    });
+                }
+                break;
 
-        case "fill_often":
-            browser.tabs.query({active: true, currentWindow: true}).then(tabInfo => {
-                fillformfields(tabInfo[0].id, "fillMostUsed");
-            });
-            break;
+            case "fill_often":
+                if (res.prefShortcutKeys['fill_often_enable']) {
+                    browser.tabs.query({active: true, currentWindow: true}).then(tabInfo => {
+                        fillformfields(tabInfo[0].id, "fillMostUsed");
+                    });
+                }
+                break;
 
-        case "clear_filled":
-            browser.tabs.query({active: true, currentWindow: true}).then(tabInfo => {
-                fillformfields(tabInfo[0].id, "clearFields");
-            });
-            break;
-    }
+            case "clear_filled":
+                if (res.prefShortcutKeys['clear_filled_enable']) {
+                    browser.tabs.query({active: true, currentWindow: true}).then(tabInfo => {
+                        fillformfields(tabInfo[0].id, "clearFields");
+                    });
+                }
+                break;
+        }
+
+    });
 }
\ No newline at end of file
