@@ -85,9 +85,9 @@ function onContextMenuItemClicked(event) {
 }
 
 function onKeyClicked(event) {
-    const keyName = event.key;
+    const keyCode = event.code;
 
-    if (keyName === 'Escape') {
+    if (keyCode === 'Escape') {
         if (WindowUtil.isModalDialogActive()) {
             WindowUtil.doCancelModalDialog();
         } else if (isModalDatetimeDialogActive()) {
@@ -102,30 +102,49 @@ function onKeyClicked(event) {
             WindowUtil.closeThisPopup();
         }
     }
+
+    // Ctrl+C Copy all
+    if (!event.altKey && event.ctrlKey && !event.shiftKey && keyCode === 'KeyC') {
+        event.preventDefault();
+        copyToClipboardAll();
+    }
+
+    // Shift+Ctrl+C Copy without formatting
+    if (!event.altKey && event.ctrlKey && event.shiftKey && keyCode === 'KeyC') {
+        event.preventDefault();
+        copyToClipboardText();
+    }
 }
 
 function copyToClipboardText() {
-    const input = document.getElementById('multiline-value');
-    const curContent = input.value;
-    let cleanContent = WindowUtil.htmlToReadableText(curContent);
-    const isDisabled = input.disabled;
-    input.disabled = false;
-    input.value = cleanContent;
-    input.select();
-    document.execCommand('copy');
-    input.value = curContent;
-    input.selectionStart = input.selectionEnd = 0;
-    input.disabled = isDisabled;
+    const input = getInputElement();
+    const cleanContent = WindowUtil.htmlToReadableText(input.value);
+    copyTextToClipboard(cleanContent);
 }
 
 function copyToClipboardAll() {
-    const input = document.getElementById('multiline-value');
-    const isDisabled = input.disabled;
-    input.disabled = false;
-    input.select();
-    document.execCommand('copy');
-    input.selectionStart = input.selectionEnd = 0;
-    input.disabled = isDisabled;
+    const input = getInputElement();
+    copyTextToClipboard(input.value);
+}
+
+function getInputElement() {
+    const type = document.getElementById("typeSelect").value;
+    const inputId = (type === 'input') ? 'value' : 'multiline-value';
+    return document.getElementById(inputId);
+}
+
+function copyTextToClipboard(value) {
+    browser.permissions.contains({permissions: ["clipboardWrite"]}).then(result => {
+        if (result) {
+            window.navigator.clipboard.writeText(value).then(function() {
+                // console.log('clipboard successfully set');
+            }, function() {
+                console.error('clipboard write failed!');
+            });
+        } else {
+            console.error('Permission clipboardWrite not available!');
+        }
+    });
 }
 
 function onOkayButton() {
@@ -403,7 +422,6 @@ function toggleMarkdownView(event) {
             tables: true,
             breaks: false,
             pedantic: false,
-            sanitize: true,
             smartLists: true,
             smartypants: true
         });
