@@ -48,47 +48,58 @@ document.addEventListener("DOMContentLoaded", function() {
 
     addStylesheetThemesToSelect();
     addMultilineSaveOptionsToSelect();
+    addUpdateIntervalOptionsToSelect();
+
+    // this sets the shortcuts keys with the current preferences (taken from commands)
+    // but not the enable state (that is not an attribute of the command)
     addShortcutKeyOptions();
 
     restoreOptions();
     document.querySelector("form").addEventListener("submit", saveOptions);
 
+    document.querySelector("#expertMode").addEventListener("change", showHideExpertPrefs);
+    document.querySelector("#expertMode").addEventListener("change", checkPropertiesChanged);
+
     document.querySelectorAll('.optionLink').forEach(link => {
         link.addEventListener("click", selectOptionSection);
     });
 
     document.querySelector("#themeSelect").addEventListener("change", themeSelectionChanged);
-    document.querySelector("#overrideAutocomplete").addEventListener("change", checkPropertiesChanged);
     document.querySelector("#dateformatSelect").addEventListener("change", checkPropertiesChanged);
+    document.querySelector("#scrollAmountSelect").addEventListener("change", checkPropertiesChanged);
+    document.querySelector("#contextMenuSelect").addEventListener("change", checkPropertiesChanged);
 
+    document.querySelector("#overrideAutocomplete").addEventListener("change", checkPropertiesChanged);
+    document.querySelector("#overrideIncognito").addEventListener("change", checkPropertiesChanged);
+    document.querySelector("#retainTypeSelect").addEventListener("change", checkPropertiesChanged);
+    document.querySelector("#retainTypeSelect").addEventListener("change", retainTypeChanged);
+    document.querySelector("#updateIntervalSelect").addEventListener("change", checkPropertiesChanged);
     document.querySelector("#versionAgeSelect").addEventListener("change", checkPropertiesChanged);
     document.querySelector("#versionLengthSelect").addEventListener("change", checkPropertiesChanged);
 
     document.querySelector("#shortcutKeysModify").addEventListener("click", showShortkeyModifySelects);
+    document.querySelector("#shortcutKeysSummary").addEventListener("click", showShortkeySummary);
+
+    document.querySelector("#autocleanup").addEventListener("change", checkPropertiesChanged);
+    document.querySelector("#keepdayshistory").addEventListener("change", checkPropertiesChanged);
+    document.querySelector("#btnCleanupNow").addEventListener("click", cleanupNow);
 
     document.querySelectorAll('input[name=radiogroupDomainlist]').forEach(radio => {
         radio.addEventListener("change", checkPropertiesChanged);
         radio.addEventListener("change", domainlistRadioChanged);
     });
-
     document.querySelector('#domainlist').addEventListener("change", domainlistChanged);
     document.querySelector('#domainListItem').addEventListener("keyup", domainlistInputChanged);
+    document.querySelectorAll('.domainbutton').forEach(btn => {btn.addEventListener("click", listButtonClicked)});
 
     document.querySelector('#fieldlist').addEventListener("change", fieldlistChanged);
     document.querySelector('#fieldListItem').addEventListener("keyup", fieldlistInputChanged);
 
-    document.querySelectorAll('.domainbutton').forEach(btn => {btn.addEventListener("click", listButtonClicked)});
-
-    document.querySelector("#autocleanup").addEventListener("change", checkPropertiesChanged);
-    document.querySelector("#keepdayshistory").addEventListener("change", checkPropertiesChanged);
-    document.querySelector("#btnCleanupNow").addEventListener("click", cleanupNow);
-
     document.querySelector("#buttonClose").addEventListener("click", closeThisPopup);
     document.addEventListener("keyup", onKeyClicked);
 
     // if update shortcut commands is not supported (chrome), hide the shortcut edit button
     if (!browser.commands.update) {
-        hideShortcutKeysModifyButton();
         showShortcutKeysModifyNotAllowedMessage();
     }
 
@@ -119,18 +130,28 @@ function closeThisPopup(event) {
 let currentOptions;
 function restoreOptions() {
     let gettingItem = browser.storage.local.get({
+        prefExpertMode           : false,
         prefInterfaceTheme       : "default",
         prefUseCustomAutocomplete: false,
+        prefSaveInIncognitoMode  : false,
         prefMultilineThresholds  : {age: "10", length: "500"},
+        prefRetainType           : "all",
+        prefUpdateInterval       : "5000",
         prefDateFormat           : "automatic",
+        prefScrollAmount         : "auto",
+        prefContextmenuAvail     : "page",
         prefShortcutKeys         : {
             // defaults here must be equal to the defaults in manifest.json
-            _execute_browser_action: OptionsUtil.getDefaultShortcutKey('_execute_browser_action'),
-            open_fhc               : OptionsUtil.getDefaultShortcutKey('open_fhc'),
-            toggle_display_fields  : OptionsUtil.getDefaultShortcutKey('toggle_display_fields'),
-            fill_recent            : OptionsUtil.getDefaultShortcutKey('fill_recent'),
-            fill_often             : OptionsUtil.getDefaultShortcutKey('fill_often'),
-            clear_filled           : OptionsUtil.getDefaultShortcutKey('clear_filled')
+            _execute_browser_action     : OptionsUtil.getDefaultShortcutKey('_execute_browser_action'),
+            open_fhc                    : OptionsUtil.getDefaultShortcutKey('open_fhc'),
+            toggle_display_fields       : OptionsUtil.getDefaultShortcutKey('toggle_display_fields'),
+            fill_recent                 : OptionsUtil.getDefaultShortcutKey('fill_recent'),
+            fill_often                  : OptionsUtil.getDefaultShortcutKey('fill_often'),
+            open_fhc_enable             : true,
+            toggle_display_fields_enable: true,
+            fill_recent_enable          : true,
+            fill_often_enable           : true,
+            clear_filled_enable         : true
         },
         prefDomainFilter         : "all",
         prefDomainList           : [],
@@ -140,14 +161,22 @@ function restoreOptions() {
     });
     gettingItem.then(res => {
         //console.log('checkbox value got from storage is [' + res.prefUseCustomAutocomplete + ']');
+        document.querySelector('#expertMode').checked = res.prefExpertMode;
         document.querySelector('#themeSelect').value = res.prefInterfaceTheme;
         document.querySelector("#overrideAutocomplete").checked = res.prefUseCustomAutocomplete;
+        document.querySelector("#overrideIncognito").checked = res.prefSaveInIncognitoMode;
         document.querySelector('#versionAgeSelect').value = res.prefMultilineThresholds.age;
         document.querySelector('#versionLengthSelect').value = res.prefMultilineThresholds.length;
+        document.querySelector('#retainTypeSelect').value = res.prefRetainType;
+        document.querySelector('#updateIntervalSelect').value = res.prefUpdateInterval;
         document.querySelector("#dateformatSelect").value = res.prefDateFormat;
+        document.querySelector("#scrollAmountSelect").value = res.prefScrollAmount;
+        document.querySelector("#contextMenuSelect").value = res.prefContextmenuAvail;
         document.querySelector("#autocleanup").checked = res.prefAutomaticCleanup;
         document.querySelector("#keepdayshistory").value = res.prefKeepDaysHistory;
 
+        checkShortcutKeyEnable(res.prefShortcutKeys);
+
         checkRadioDomainByValue(res.prefDomainFilter);
 
         setListOptions("#domainlist", res.prefDomainList);
@@ -159,6 +188,8 @@ function restoreOptions() {
         domainlistRadioChanged();
         domainlistChanged();
         fieldlistChanged();
+        retainTypeChanged();
+        showHideExpertPrefs();
 
         currentOptions = Object.assign({}, res);
         checkPropertiesChanged();
@@ -176,9 +207,14 @@ function saveOptions(e) {
         eventType: 888,
         interfaceThemeChanged:       (currentOptions.prefInterfaceTheme !== newOptions.prefInterfaceTheme),
         overrideAutocompleteChanged: (currentOptions.prefUseCustomAutocomplete !== newOptions.prefUseCustomAutocomplete),
+        overrideIncognitoChanged:    (currentOptions.prefSaveInIncognitoMode !== newOptions.prefSaveInIncognitoMode),
         multilineThresholdsChanged:  (currentOptions.prefMultilineThresholds.age !== newOptions.prefMultilineThresholds.age
                                    || currentOptions.prefMultilineThresholds.length !== newOptions.prefMultilineThresholds.length),
+        retainTypeChanged:           (currentOptions.prefRetainType !== newOptions.prefRetainType),
+        updateIntervalChanged:       (currentOptions.prefUpdateInterval !== newOptions.prefUpdateInterval),
         dateFormatChanged:           (currentOptions.prefDateFormat !== newOptions.prefDateFormat),
+        scrollAmountChanged:         (currentOptions.prefScrollAmount !== newOptions.prefScrollAmount),
+        contextmenuAvailChanged:     (currentOptions.prefContextmenuAvail !== newOptions.prefContextmenuAvail),
         domainFilterChanged:         (currentOptions.prefDomainFilter !== newOptions.prefDomainFilter || !arrayContentEquals(currentOptions.prefDomainList, newOptions.prefDomainList)),
         fieldFilterChanged:          !arrayContentEquals(currentOptions.prefFieldList, newOptions.prefFieldList)
     };
@@ -202,11 +238,17 @@ function saveOptions(e) {
 
 function getNewOptions() {
     return {
+        prefExpertMode           : document.querySelector("#expertMode").checked,
         prefInterfaceTheme       : document.querySelector("#themeSelect").value,
         prefUseCustomAutocomplete: document.querySelector("#overrideAutocomplete").checked,
+        prefSaveInIncognitoMode  : document.querySelector("#overrideIncognito").checked,
         prefMultilineThresholds  : {age   : document.querySelector("#versionAgeSelect").value,
                                     length: document.querySelector("#versionLengthSelect").value},
+        prefRetainType           : document.querySelector("#retainTypeSelect").value,
+        prefUpdateInterval       : document.querySelector("#updateIntervalSelect").value,
         prefDateFormat           : document.querySelector("#dateformatSelect").value,
+        prefScrollAmount         : document.querySelector("#scrollAmountSelect").value,
+        prefContextmenuAvail     : document.querySelector("#contextMenuSelect").value,
         prefShortcutKeys         : getAllShortcutKeyValues(),
         prefDomainFilter         : getCheckedRadioDomainValue(),
         prefDomainList           : getList("#domainlist"),
@@ -235,6 +277,54 @@ function selectOptionSection(event) {
     currentLinkElm.classList.add("selected");
 }
 
+function retainTypeChanged() {
+    const retainType = document.querySelector("#retainTypeSelect").value;
+
+    const selAge = document.querySelector("#versionAgeSelect");
+    const selAgeLbl = document.querySelector("#versionAgeSelectLabel");
+    const selLen = document.querySelector("#versionLengthSelect");
+    const selLenLbl = document.querySelector("#versionLengthSelectLabel");
+
+    if (retainType === 'single') {
+        // disable multiline options
+        selAgeLbl.classList.add("disabled");
+        selAge.setAttribute("disabled", "true");
+        selLenLbl.classList.add("disabled");
+        selLen.setAttribute("disabled", "true");
+    } else {
+        selAgeLbl.classList.remove("disabled");
+        selAge.removeAttribute("disabled");
+        selLenLbl.classList.remove("disabled");
+        selLen.removeAttribute("disabled");
+    }
+}
+
+function showHideExpertPrefs() {
+    const expertModeChecked = document.querySelector("#expertMode").checked;
+
+    Array.from(document.getElementsByClassName('expert-pref')).forEach(elem => {
+        if (expertModeChecked) {
+            elem.style.display = 'unset';
+        } else {
+            elem.style.display = 'none';
+        }
+    });
+
+    // check if the current selected category is still visible
+    const oldLinkElm = document.querySelector(".optionLink.selected");
+    if (oldLinkElm && oldLinkElm.offsetParent === null) {
+        // unselect the now hidden fieldset
+        const oldFieldsetId = "fld_" + oldLinkElm.id;
+        const oldFldSet = document.querySelector("#" + oldFieldsetId);
+        oldFldSet.style.display = "none";
+        oldLinkElm.classList.remove("selected");
+
+        // select the first visible fieldset
+        document.querySelector('#display').classList.add('selected');
+        document.querySelector('#fld_display').style.display = 'block';
+    }
+}
+
 function domainlistRadioChanged() {
     const radioAllowAllChecked = document.querySelector("#radioDomainlistAll").checked;
 
@@ -277,6 +367,12 @@ function fieldlistInputChanged() {
     setListButtonsState("#fieldlist", "#fieldListItem", "#fieldAdd", "#fieldModify", "#fieldDelete");
 }
 
+function shortcutKeyEnableChanged(event) {
+    const commandName = event.target.getAttribute('data-cmd');
+    shortcutKeyCommandEnableChanged(commandName);
+    checkPropertiesChanged();
+}
+
 function checkPropertiesChanged() {
     // enable apply button only if properties have changed
     let changed = false;
