@@ -13,7 +13,7 @@ function receiveEvents(fhcEvent, sender, sendResponse) {
     if (fhcEvent.eventType) {
         switch (fhcEvent.eventType) {
             case 1:
-                if (!blockedByFilter(fhcEvent)) {
+                if (!blockedByFilter(fhcEvent) && doRetainFieldType(fhcEvent)) {
                     // Process Text
                     fhcEvent.value = JSON.parse(fhcEvent.value);
                     //console.log("Received a content event for " + fhcEvent.id + " content is: " + fhcEvent.value);
@@ -93,7 +93,7 @@ function receiveEvents(fhcEvent, sender, sendResponse) {
 
             case 888:
                 // console.log('Background script: options changed!');
-                if (fhcEvent.domainFilterChanged || fhcEvent.fieldFilterChanged) {
+                if (fhcEvent.domainFilterChanged || fhcEvent.fieldFilterChanged || fhcEvent.retainTypeChanged || fhcEvent.overrideIncognitoChanged) {
                     initFilterOptions();
                 }
                 if (fhcEvent.multilineThresholdsChanged) {
@@ -273,12 +273,24 @@ function initFilterOptions() {
     });
 }
 
+function doRetainFieldType(fhcEvent) {
+    if (fhcEvent.type === 'input') {
+        return OptionsUtil.doRetainSinglelineField(filterPrefs);
+    } else {
+        return OptionsUtil.doRetainMultilineField(filterPrefs);
+    }
+}
+
 function blockedByFilter(fhcEvent) {
+    if (fhcEvent.incognito && !OptionsUtil.doSaveInIncognitoMode(filterPrefs)) {
+        //console.log('name:' + fhcEvent.name + ' is blocked in Incognito mode');
+        return true;
+    }
     if (OptionsUtil.isDomainBlocked(fhcEvent.host, filterPrefs)) {
         //console.log('type:' + fhcEvent.type + ' name:' + fhcEvent.name + ' host:' + fhcEvent.host + ' is blocked by domain');
         return true;
     }
-    if (fhcEvent.eventType === 1 && OptionsUtil.isTextFieldBlocked(fhcEvent.name, filterPrefs)) {
+    if (fhcEvent.eventType === 1 && OptionsUtil.isTextFieldBlocked(fhcEvent.host, fhcEvent.name, filterPrefs)) {
         //console.log('type:' + fhcEvent.type + ' name:' + fhcEvent.name + ' host:' + fhcEvent.host + ' is blocked by field');
         return true;
     }
