@@ -26,11 +26,18 @@ $(document).ready(function() {
     const tableElement = $('#fhcTable');
     let table;
 
-    // const table = createDataTable(tableElement);
-    OptionsUtil.getDateFormat(
-    ).then(dateformat => {
+    browser.storage.local.get({
+        pageSizeBig: 500,
+        prefDateFormat: 'automatic',
+        prefScrollAmount: 'auto',
+        prefColBigVisible: []
+    }).then(result => {
+        let pageSizeBig = result.pageSizeBig;
+        let dateformat = result.prefDateFormat;
+        let scrollAmount = result.prefScrollAmount;
+        const prefColBigVisible = OptionsUtil.initColPrefs(result.prefColBigVisible);
         DataTableUtil.dateformat = dateformat;
-        table = createDataTable(tableElement, dateformat);
+        table = createDataTable(tableElement, dateformat, scrollAmount, prefColBigVisible);
 
         // add event listener for saving changed pageSize
         table.on('length.dt', function(e, settings, len) {
@@ -41,23 +48,33 @@ $(document).ready(function() {
             // console.log( 'New page length: '+len);
         });
 
-        // Add event listener for opening and closing details
+        // add event listener for saving changed column visibility
+        table.on('column-visibility.dt', function(e, settings, column, state) {
+            if (column > 1) {
+                // console.log('Column '+ column +' has changed to '+ (state ? 'visible' : 'hidden'));
+                prefColBigVisible[column - 2] = state;
+                browser.storage.local.set({
+                    prefColBigVisible: prefColBigVisible
+                });
+            }
+        });
+
+        // add event listener for opening and closing details
         tableElement.find('tbody').on('click', 'td.my-details-control', function() {
             DataTableUtil.openDetailViewOnRowClick($(this), table, "view");
         });
-
         tableElement.find('tbody').on('dblclick', 'tr', function() {
             DataTableUtil.openDetailViewOnRowClick($(this), table, "view");
         });
 
-        // Add event listener for select events
+        // add event listener for select events
         table.on('select', function (e, dt, type /*, indexes */) {
             if (type === 'row') {
                 selectionChangedHandler();
             }
         });
 
-        // Add event listener for deselect events
+        // add event listener for deselect events
         table.on('deselect', function (e, dt, type /*, indexes*/) {
             if (type === 'row') {
                 selectionChangedHandler();
@@ -82,26 +99,22 @@ $(document).ready(function() {
                 let row = table.row( tr );
                 dataRightClicked = row.data();
                 WindowUtil.showContextMenu(event, '#content');
-            }).on('click', 'tr', function(event) {
-            // Event listener for closing the context menu when clicked outside the menu
-            WindowUtil.hideContextMenuOnClick(event);
-        });
-    }).then(() => {
-        browser.storage.local.get(
-            {pageSizeBig: 500}
-        ).then(result => {
-                // set the pagesize to the last used value
-                table.page.len(result.pageSizeBig);
-
-                // populate tableview with data from the database
-                populateViewFromDatabase(table, 25, null, null);
-                selectionChangedHandler();
-            },
-            () => {
-                console.error("Get last used pagesize error", this.error);
+            })
+            .on('click', 'tr', function(event) {
+                // Event listener for closing the context menu when clicked outside the menu
+                WindowUtil.hideContextMenuOnClick(event);
             }
         );
-    });
+
+        // set the pagesize to the last used value
+        table.page.len(pageSizeBig);
+
+        // populate tableview with data from the database
+        populateViewFromDatabase(table, 25, null, null);
+        selectionChangedHandler();
+      },
+      () => {console.error("Get preferences error", this.error);}
+    );
 
     // Prevent the default right-click contextmenu
     document.oncontextmenu = function() {return false;};
@@ -120,7 +133,7 @@ $(document).ready(function() {
         onMenuClicked(event.currentTarget.id);
     });
 
-    // add keyhandler for menu
+    // add keyhandler for esc/del/select-all and menu
     $('body').on('keyup', function(event) {
         onKeyClicked(event);
     });
@@ -150,22 +163,34 @@ $(document).ready(function() {
     setInterval(updateTableRowsAgeColumn, 60*1000);
 });
 
-function createDataTable(tableElement, dateformat) {
+function createDataTable(tableElement, dateformat, scrollAmount, prefColVisible) {
     let languageURL = DataTableUtil.getLanguageURL();
     const i18nFld = DataTableUtil.getLocaleFieldNames();
     const i18nAll = browser.i18n.getMessage("pagingAll") || 'All';
+    const i18nColVis = browser.i18n.getMessage("buttonColumnVisibility") || 'Column visibility';
+    const i18nColVisRestore = browser.i18n.getMessage("buttonRestoreColumnVisibility") || 'Restore visibility';
 
     return tableElement.DataTable( {
+        dom: 'lBfrtip',
         responsive: {details: false},
         scrollY: '300px',
-        language: {url: languageURL},
+        language: {
+            url: languageURL,
+            buttons: {
+                colvisRestore: i18nColVisRestore,
+                colvis: i18nColVis
+            }
+        },
         paging: true,
         lengthMenu: [[100, 500, 1000, 2000, -1], [100, 500, 1000, 2000, i18nAll]],
         pageLength: 500,
         fnDrawCallback: function(){
             $('.dataTables_scrollBody').mCustomScrollbar({
-                showArrows: 'true',
-                scrollButtons:{ enable: true },
+                scrollInertia: 500,
+                showArrows: true,
+                scrollButtons:{ enable: true, scrollAmount: 5 },
+                mouseWheel:{ scrollAmount: scrollAmount },
+                keyboard:{ enable: true, scrollAmount: 5 },
                 theme: "3d-thick-dark"
             });
         },
@@ -175,6 +200,11 @@ function createDataTable(tableElement, dateformat) {
             selector: 'td:not(.my-details-control)'
         },
         order: [[ 7, "desc" ]],
+        buttons: [{
+            extend: 'colvis',
+            columns: ':gt(1)',
+            postfixButtons: [ 'colvisRestore' ]
+        }],
         columns: [
             {
                 responsivePriority: 1,
@@ -192,7 +222,8 @@ function createDataTable(tableElement, dateformat) {
             { title: i18nFld.last, responsivePriority: 7 },
             { title: i18nFld.age, responsivePriority: 6 },
             { title: i18nFld.host, responsivePriority: 8 },
-            { title: i18nFld.uri, responsivePriority: 11 }
+            { title: i18nFld.uri, responsivePriority: 11 },
+            { title: i18nFld.length, responsivePriority: 12 }
         ],
         columnDefs: [
             {
@@ -202,6 +233,7 @@ function createDataTable(tableElement, dateformat) {
             },
             {
                 targets: 2,
+                visible: prefColVisible[0],
                 data: 1,
                 className: "dt-head-left",
                 render: function ( data, type /*, full, meta */) {
@@ -210,6 +242,7 @@ function createDataTable(tableElement, dateformat) {
             },
             {
                 targets: 3,
+                visible: prefColVisible[1],
                 data: 2,
                 className: "dt-head-left",
                 render: function ( data, type /* , full, meta */) {
@@ -218,11 +251,13 @@ function createDataTable(tableElement, dateformat) {
             },
             {
                 targets: 4,
+                visible: prefColVisible[2],
                 data: 3,
                 className: "dt-head-left"
             },
             {
                 targets: 5,
+                visible: prefColVisible[3],
                 data: 4,
                 type: "num",
                 searchable: false,
@@ -233,6 +268,7 @@ function createDataTable(tableElement, dateformat) {
             },
             {
                 targets: 6,
+                visible: prefColVisible[4],
                 data: 5,
                 className: "dt-head-left",
                 render: function ( data, type /*, full, meta */) {
@@ -241,6 +277,7 @@ function createDataTable(tableElement, dateformat) {
             },
             {
                 targets: 7,
+                visible: prefColVisible[5],
                 data: 6,
                 className: "dt-head-left",
                 render: function ( data, type /*, full, meta */) {
@@ -249,6 +286,7 @@ function createDataTable(tableElement, dateformat) {
             },
             {
                 targets: 8,
+                visible: prefColVisible[6],
                 data: 6,
                 className: "dt-head-left",
                 render: function ( data, type /*, full, meta */) {
@@ -257,6 +295,7 @@ function createDataTable(tableElement, dateformat) {
             },
             {
                 targets: 9,
+                visible: prefColVisible[7],
                 data: 7,
                 className: "dt-head-left",
                 render: function ( data, type /*, full, meta */) {
@@ -265,11 +304,23 @@ function createDataTable(tableElement, dateformat) {
             },
             {
                 targets: 10,
+                visible: prefColVisible[8],
                 data: 8,
                 className: "dt-head-left",
                 render: function ( data, type /*, full, meta */) {
                     return DataTableUtil.ellipsis(data, type, 30, false, true);
                 }
+            },
+            {
+                targets: 11,
+                visible: prefColVisible[9],
+                data: 2,
+                type: "num",
+                searchable: false,
+                className: "dt-right",
+                render: function ( data, /*type, full, meta */) {
+                    return (!data) ? "0" : data.length;
+                }
             }
         ]
     });
@@ -278,7 +329,7 @@ function createDataTable(tableElement, dateformat) {
 
 function selectAll() {
     let table = $('#fhcTable').DataTable();
-    table.rows().select();
+    table.rows( {order:'index', search:'applied'} ).select();
 }
 
 function selectNone() {
@@ -293,7 +344,6 @@ function selectInvert() {
     table.rows(curSelected).deselect();
 }
 
-
 function deleteSelectedItemsAsk() {
     let rows = $('#fhcTable').DataTable().rows('.selected');
 
@@ -383,6 +433,7 @@ function selectionChangedHandler() {
     setMenuItemEnabled('delete', (noSelected !== 0));
     setMenuItemEnabled('modify', (noSelected !== 0));
     setMenuItemEnabled('copy2clipboard', (noSelected === 1));
+    setMenuItemEnabled('copy2clipboardText', (noSelected === 1));
 }
 
 function setButtonEnabled(id, enabled) {
@@ -433,18 +484,34 @@ function onContextMenuClicked(menuItemId) {
             break;
 
         case "modify-ctx":
-            editEntry(dataRightClicked);
+            // if one or more entries are selected edit the selection, otherwise edit the right-clicked item
+            if (haveSelectedItems()) {
+                editSelectedEntries();
+            } else {
+                editEntry(dataRightClicked);
+            }
             WindowUtil.hideContextMenu();
             break;
 
         case "delete-ctx":
-            DataTableUtil.deleteItemFromDatabase(dataRightClicked[0]);
-            DataTableUtil.broadcastItemDeletedFromDatabase();
+            // if one or more entries are selected delete selection, otherwise delete the right-clicked item
+            if (haveSelectedItems()) {
+                deleteSelectedItemsAsk();
+            } else {
+                DataTableUtil.deleteItemFromDatabase(dataRightClicked[0]);
+                DataTableUtil.broadcastItemDeletedFromDatabase();
+            }
             WindowUtil.hideContextMenu();
             break;
 
         case "copy2clipboard-ctx":
-            DataTableUtil.copyEntryToClipboard(dataRightClicked);
+            // if one or more entries are selected copy the first selected item, otherwise copy the right-clicked item
+            if (haveSelectedItems()) {
+                const dataRow1 = $('#fhcTable').DataTable().rows('.selected').data()[0];
+                DataTableUtil.copyDataToClipboard(dataRow1);
+            } else {
+                DataTableUtil.copyDataToClipboard(dataRightClicked);
+            }
             WindowUtil.hideContextMenu();
             break;
 
@@ -503,6 +570,13 @@ function onMenuClicked(menuItemId) {
             }
             break;
 
+        case "copy2clipboardText":
+            if (isMenuItemEnabled(menuItemId)) {
+                DataTableUtil.copySelectedEntryCleanToClipboard($('#fhcTable').DataTable());
+                WindowUtil.showModalInformation({titleId:'dialogInformationTitle', msgId:'informClipboardValueCopied'});
+            }
+            break;
+
         case "selectall":
             selectAll();
             break;
@@ -559,24 +633,70 @@ function onMenuClicked(menuItemId) {
 
 function onKeyClicked(event) {
     const keyName = event.key;
+    const keyCode = event.code;
 
-    if (keyName === 'Escape' && WindowUtil.isModalDialogActive()) {
-        WindowUtil.doCancelModalDialog();
+    if (keyCode === 'Escape') {
+        if (WindowUtil.isModalDialogActive()) {
+            WindowUtil.doCancelModalDialog();
+            return;
+        }
+        if (WindowUtil.isContextMenuShown()) {
+            WindowUtil.hideContextMenu();
+            return;
+        }
+    }
+
+    // Select all (Ctrl-A seems locale independent)
+    if (!event.altKey && event.ctrlKey && !event.shiftKey && keyCode === 'KeyA') {
+        // event.stopImmediatePropagation not effective, user-select style none will prevent showing selected elements
+        selectAll();
         return;
     }
 
-    if (event.altKey && !event.ctrlKey && !event.shiftKey && isAlpha(keyName)) {
-        // console.log("We have an Alt-key event: " + keyName);
+    // Delete and Shift+Delete key
+    if (!event.altKey && !event.ctrlKey && keyCode === 'Delete') {
+        if (isMenuItemEnabled('delete')) {
+            if (event.shiftKey) {
+                deleteSelectedItems();
+            } else {
+                deleteSelectedItemsAsk();
+            }
+        }
+        return;
+    }
 
+    // Ctrl+C Copy all
+    if (!event.altKey && event.ctrlKey && !event.shiftKey && keyCode === 'KeyC') {
+        event.preventDefault();
+
+        // if one or more entries are selected copy the first selected item
+        if (haveSelectedItems()) {
+            const dataRow1 = $('#fhcTable').DataTable().rows('.selected').data()[0];
+            DataTableUtil.copyDataToClipboard(dataRow1);
+        }
+    }
+
+    // Shift+Ctrl+C Copy without formatting
+    if (!event.altKey && event.ctrlKey && event.shiftKey && keyCode === 'KeyC') {
+        event.preventDefault();
+
+        // if one or more entries are selected copy the first selected item
+        if (haveSelectedItems()) {
+            const dataRow1 = $('#fhcTable').DataTable().rows('.selected').data()[0];
+            DataTableUtil.copyDataCleanToClipboard(dataRow1);
+        }
+    }
+
+    // context menu shortcut (Alt+key)
+    if (event.altKey && !event.ctrlKey && !event.shiftKey && isAlpha(keyName)) {
         // try to find a matching menu-item
         const menuItems = $("span[data-access-key='" + keyName +"']");
         if (menuItems.length > 0) {
             const menuItem = menuItems[0];
 
             // if no id its a toplevel menu
-            if (!menuItem.id) {
-                // $(menuItem).parent().addClass('hovered');
-                // $(menuItem).parent().find('*').addClass('hovered');
+            if (menuItem.id) {
+                onMenuClicked(menuItem.id);
             }
         }
     }
